/*             This file is part of the New World OS project
--                 Copyright (C) 2006-2007  QRW Software
--           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
--                      http://www.qrwsoftware.com
--                      http://nwos.sourceforge.com
--
-- NWOS is free software;  you can redistribute it and/or modify it under the
-- terms of the GNU General Public License  as published by the Free Software
-- Foundation; either version 2, or (at your option) any later version.  This
-- software is distributed with the hope that it will be useful,  but WITHOUT
-- ANY WARRANTY;  without  even the  implied warranty  of MERCHANTABILITY  or
-- FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License for
-- more  details.  You should have received a copy  of the GNU General Public
-- License along with this package;  see the file LICENSE.  If not, write to:
--
--      Free Software Foundation, Inc.
--      59 Temple Place - Suite 330
--      Boston, MA 02111-1307, USA.
--
-- $Log: log_disc.c,v $
-- Revision 1.22  2007/06/13 21:15:01  jsedwards
-- Comment out debugging print statement accidentally left in.
--
-- Revision 1.21  2007/06/13 20:53:50  jsedwards
-- Changed to take a directory as a parameter instead of a list of files.
--
-- Revision 1.20  2007/06/04 13:44:05  jsedwards
-- Add 'match' paramter to find_matchine_path_and_file_association.
--
-- Revision 1.19  2007/06/03 22:03:55  jsedwards
-- Changed so that 'location' can be a Location object or a Person object.
--
-- Revision 1.18  2007/04/28 12:40:56  jsedwards
-- Added printing of each file name so the printing from finding the name and
-- file isn't so weird.
--
-- Revision 1.17  2007/02/11 16:58:26  jsedwards
-- Changed so DEFAULT_TYPE has to specify RO (Read-Only) or RW (Read-Write).
--
-- Revision 1.16  2007/01/27 01:43:51  jsedwards
-- Change storage options to be numbered from 1 to 6 instead of 0 to 5.
--
-- Revision 1.15  2007/01/20 13:36:58  jsedwards
-- Change encryption level to "very low".
--
-- Revision 1.14  2007/01/13 14:36:48  jsedwards
-- Added "failed" storage location for disk copies that didn't write, verify,
-- or some other problem occured.
--
-- Revision 1.13  2007/01/10 13:15:59  jsedwards
-- Added "missing" as a storage location for discs that are misplaced.
--
-- Revision 1.12  2007/01/07 19:13:41  jsedwards
-- Add call to log the arguments.
--
-- Revision 1.11  2007/01/06 20:12:06  jsedwards
-- Changed binder names and error if no files specified on command line.
--
-- Revision 1.10  2007/01/05 13:51:16  jsedwards
-- Change to work with new file association.
--
-- Revision 1.9  2006/12/07 14:10:25  jsedwards
-- Removed call to file_setup, no longer needed.
--
-- Revision 1.8  2006/12/02 17:23:32  jsedwards
-- Removed start_time that wasn't used and added SafeDepositBox location.
--
-- Revision 1.7  2006/11/19 16:40:22  jsedwards
-- Moved creation of disc list to separate program (disc_list) and made this
-- program just verify the disc list create the disc_copy object.
--
-- Revision 1.6  2006/11/18 14:34:58  jsedwards
-- Changed so that discs can be stored in one of three different binders.
--
-- Revision 1.5  2006/11/11 12:01:04  jsedwards
-- Update e-mail address to something that works.
--
-- Revision 1.4  2006/11/07 14:03:35  jsedwards
-- Add assert to verify file_path didn't come back void.
--
-- Revision 1.3  2006/11/06 13:54:07  jsedwards
-- Changed wording in print statement because disc_list now stores the
-- references to the file_path object instead of the file object.
--
-- Revision 1.2  2006/11/05 21:32:18  jsedwards
-- Add code to create storage_location and disc_copy objects.
--
-- Revision 1.1  2006/11/04 18:56:22  jsedwards
-- Program to scan all the files on a CD or DVD and make a disk_list object
-- of it.
--
*/


#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "objectify.h"


static void get_input(char* descr, char* buffer, size_t size)
{
    char *ptr;

    while (1)
    {
	printf("%s: ", descr);
	fflush(stdout);
	fgets(buffer, size, stdin);
	ptr = strchr(buffer, '\n');
	if (ptr != NULL)
	{
	    *ptr = '\0';
	    break;
	}
	do { fgets(buffer, size, stdin); } while (strchr(buffer, '\n') == NULL);
	printf("input too long - try again!\n");
    }
}



int main(int argc, char* argv[])
{
    ObjRef root_object_ref;
    uint8 big_key[16 + 8 + 4];
    uint8 bf_key[16];
    uint32 linear;
    uint32 serial;
    ObjRef ref;
    int i;
    char id[13];
    char copy_num[3] = "";
    char binder_num[3] = "";
    char name[24];
    char *p;
    bool ok = false;
    int binder;
    char* file_names[MAX_FILES_PER_DISC_LIST];
    int num_files;
    ObjRef files[MAX_FILES_PER_DISC_LIST];
    ObjRef location_ref[6];
    ObjRef disc_list_ref;
    ObjRef person_ref;
    bool missing_files = false;
#if 0
    bool bad_checksum = false;
    CheckFileResult check_file_result;
#endif
    uint8 kludge[MAX_SIZE_DISC_LIST];
    C_struct_Disc_List* ptr_list_obj = (C_struct_Disc_List*)kludge;
    DIR* dp;
    struct dirent *dir_entry;


    if (argc != 2)
    {
	fprintf(stderr, "usage: %s directory\n", argv[0]);
	exit(1);
    }

    dp = opendir(argv[1]);
    if (dp == NULL)
    {
	perror(argv[1]);
	exit(1);
    }

    num_files = 0;
    dir_entry = readdir(dp);
    while (dir_entry != NULL)
    {
	if (strcmp(dir_entry->d_name, ".") != 0 && strcmp(dir_entry->d_name, "..") != 0)
	{
	    if (num_files == MAX_FILES_PER_DISC_LIST)
	    {
		fprintf(stderr, "Too many files in directory: %d, currently can only handle %d files max\n",
			num_files, MAX_FILES_PER_DISC_LIST);
		exit(1);
	    }

	    file_names[num_files] = malloc(strlen(dir_entry->d_name) + 1);

	    if (file_names[num_files] == NULL)
	    {
		perror("allocating memory for file name");
		exit(1);
	    }

	    strcpy(file_names[num_files], dir_entry->d_name);

	    /* printf("%s\n", file_names[num_files]); */

	    num_files++;
	}

	dir_entry = readdir(dp);
    }

    if (closedir(dp) != 0)
    {
	perror(argv[1]);
	exit(1);
    }

    printf("\n");

    nwos_log_arguments(argc, argv);

    nwos_get_key_from_password(big_key, sizeof(big_key));

    memcpy(bf_key, big_key, 16);
    linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
    memcpy(root_object_ref.id, big_key+20, 4);
    serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];

    nwos_initialize_objectify(bf_key, linear, serial, DEFAULT_TYPE_RW, DEFAULT_FILE);

    nwos_set_root_object(&root_object_ref);

    nwos_set_encryption_level(Encryption_Extreme);

    nwos_create_storage_location("Backup Binder", &location_ref[0]);
    nwos_create_storage_location("ISO Binder", &location_ref[1]);
    nwos_create_storage_location("Music Binder", &location_ref[2]);
    nwos_create_storage_location("Safe Deposit Box", &location_ref[3]);
    nwos_create_storage_location("Missing", &location_ref[4]);
    nwos_create_storage_location("Failed", &location_ref[5]);

    nwos_set_encryption_level(Encryption_Very_Low);


    /* first verify that all the listed files are already in the system */

    memset(files, 0, sizeof(files));

    for (i = 0; i < num_files; i++)
    {
	printf("%s: ", file_names[i]);
	fflush(stdout);

	if (!nwos_find_matching_path_and_file_association(argv[1], file_names[i], &files[i], MatchAll))
	{
	    missing_files = true;
	    printf("  <--- NOT found");
	}

	printf("\n");
    }

    if (missing_files)
    {
	printf("The following files are not in the system:\n\n");

	for (i = 0; i < num_files; i++)
	{
	    if (is_void_reference(&files[i]))
	    {
		printf("   %s\n", file_names[i]);
	    }
	}

	printf("\n");
	exit(1);
    }


    /* find and verify the disc */

    if (!nwos_find_matching_disc_list(files, &disc_list_ref))
    {
	printf("No matching disc list found!\n");
	exit(1);
    }

    nwos_read_variable_sized_object_from_disk(&disc_list_ref, kludge, sizeof(kludge), &nwos_get_disc_list_object_size);

    memcpy(id, ptr_list_obj->id, sizeof(id) - 1);
    id[sizeof(id) - 1] = '\0';

    printf("disc list %02x%02x%02x%02x: %s\n",
	   disc_list_ref.id[0], disc_list_ref.id[1], disc_list_ref.id[2], disc_list_ref.id[3], id);


    /* create record of this disc */

    ok = false;
    while (!ok)
    {
	printf("Copy number: ");
	fflush(stdout);

	fgets(copy_num, sizeof(copy_num), stdin);

	p = strchr(copy_num, '\n');   /* find the newline char */

	if (p == NULL)    /* line was tool long */
	{
	    while (p == NULL) 
	    {
		fgets(copy_num, sizeof(copy_num), stdin);
		p = strchr(copy_num, '\n');   /* find the newline char */
	    }
	    printf("copy number can only be 1 digit in this version\n");
	}
	else     /* line was ok */
	{
	    *p = '\0';   /* eliminate the newline character */

	    if (strlen(copy_num) != 1)
	    {
		printf("copy number can only be 1 digit in this version\n");
	    }
	    else if (copy_num[0] < '1' || copy_num[0] > '9')
	    {
		printf("copy number must be a 1 digit number in this version\n");
	    }
	    else
	    {
		ok = true;   /* we should be good to go */
	    }
	}
    }

    ok = false;
    while (!ok)
    {
	printf("Binder (1=Backups, 2=ISOs, 3=Music, 4=Safe Deposit Box 5=Missing 6=Failed 7=person): ");
	fflush(stdout);

	fgets(binder_num, sizeof(binder_num), stdin);

	p = strchr(binder_num, '\n');   /* find the newline char */

	if (p == NULL)    /* line was tool long */
	{
	    while (p == NULL) 
	    {
		fgets(binder_num, sizeof(binder_num), stdin);
		p = strchr(binder_num, '\n');   /* find the newline char */
	    }
	    printf("copy number can only be 1 digit in this version\n");
	}
	else     /* line was ok */
	{
	    *p = '\0';   /* eliminate the newline character */

	    if (strlen(binder_num) != 1)
	    {
		printf("storage binder can only be 1 digit\n");
	    }
	    else if (binder_num[0] < '1' || binder_num[0] > '7')
	    {
		printf("storage binder must be 1 to 7\n");
	    }
	    else
	    {
		ok = true;   /* we should be good to go */
	    }
	}
    }


    binder = binder_num[0] - '1';

    assert(0 <= binder && binder <= 6);

    if (binder == 6)
    {
	get_input("name", name, sizeof(name));

	if (nwos_find_person(name, &person_ref))
	{
	    assert(nwos_create_disc_copy(&disc_list_ref, copy_num[0] - '0', &person_ref, &ref) == CREATED_NEW);
	}
	else
	{
	    fprintf(stderr, "%s not found, terminating without creating record\n", name);
	}
    }
    else
    {
	assert(nwos_create_disc_copy(&disc_list_ref, copy_num[0] - '0', &location_ref[binder], &ref) == CREATED_NEW);
    }

    printf("disc copy: %02x%02x%02x%02x\n", ref.id[0], ref.id[1], ref.id[2], ref.id[3]);

    for (i = 0; i < num_files; i++)
    {
	free(file_names[i]);
    }

    nwos_terminate_objectify();

    return 0;
}


